#include <stdio.h>
#include <fcntl.h>

#include <asm/types.h>   /* headers for architecture specific data types                         */
#include <unistd.h>      /* contains the prototype for i386 specific ioperm functioni            */
#include <asm/io.h>      /* obviously architecture specific (asm directory) for inb, outb macros */
#include <string.h>

#define LP_BASE 0x240
#define FONTSIZE 5
#define STARTASCII 65

__u8 fontarray[50][FONTSIZE];
int fd;

void readfonts(char *filename) {
  char line[10];
  __u8 column;
  int index = 0, colnum = 0, i;
  FILE *fonts = fopen(filename,"r");
  while (fgets(line, 10, fonts) != NULL) {
    if (colnum == FONTSIZE) {
      index++;
      colnum = 0;
    }

    column = 0;
    for (i = 0; i < strlen(line); i++) {
      if (line[i] == '+') {
	column |= (__u8)(1 << i);
      }
    }
    fontarray[index][colnum] = column;
    colnum++;
  }
  fclose(fonts);
}

void diplayascii(char ascii, int dir, int delay) 
{
  int i;
  for (i = 0; i < FONTSIZE; i++) {
    if (dir == 1) {
      write(fd,&fontarray[ascii-STARTASCII][i],1);
      //      outb(fontarray[ascii - STARTASCII][i], LP_BASE);
      //      printf("%X \n",~ fontarray[ascii - STARTASCII][i]);
    } else {
      write(fd,&fontarray[ascii-STARTASCII][FONTSIZE - i - 1],1);
      //      outb(fontarray[ascii - STARTASCII][FONTSIZE - i - 1], LP_BASE);
      //      printf("%X \n",~ fontarray[ascii - STARTASCII][FONTSIZE - i - 1]);
    }
    //    usleep(delay);
  }
}

int main(int argc, char** argv) {

	__u8 portdata;
	int i,j;
	int index = 0;
        int dir = 1;
	fd = open("/dev/ibus0",O_RDWR);

	if (argc == 4) {
	  readfonts(argv[1]);
	} else {
	  printf("No fonts file. \n");
	  exit(1);
	}

	/* attempt to reserve region in io space */
	if(ioperm(LP_BASE, 3, 1) < 0) {
		fprintf(stderr, "Cannot reserve region in io space, aborting...\n");
		exit(1);
	}

	/* Print the font array
	for (i = 0; i<3; i++) {
	  for (j = 0; j<FONTSIZE; j++) {
	    printf("%X \n",fontarray[i][j]);
	  }
	}
	*/

	while (1) {
	  if (index == strlen(argv[2]) || index < 0) {
	    dir = (dir == 1? -1 : 1);
	    index += dir;
	  }
	  diplayascii(argv[2][index], dir, atol(argv[3]));
	  index += dir;
	}
        
	/* send data byte to io port */
	portdata = inb(LP_BASE);

	printf("Eight bit value returned form port: %d\n", portdata);

	/* attempt to release reserved region in io space */
	if(ioperm(LP_BASE, 3, 0) < 0) {
		fprintf(stderr, "Cannot release region in io space, aborting...\n");
		exit(1);
	}

	return(0);  /* just to be nice ;) */

}
