/*
 * irq.c -- demonstrate basic irq functionality in the kernel
 */

#include <linux/module.h>

#include <linux/kernel.h> /* printk() */
#include <linux/errno.h>  /* error codes */
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/tqueue.h>
#include <linux/interrupt.h>

#include <asm/io.h>

#define LED_BASE    0x240
#define IRQ_LED_ON  9
#define IRQ_LED_OFF 10

static int led_on = 0;

int irq_read_proc(char *buf, char **start, off_t offset, int len, int unused) {
  len = sprintf(buf, "Fill in with you desired information...\n");
  return len;
}

struct proc_dir_entry irq_proc_entry = {
  0,
  3, "irq",
  S_IFREG | S_IRUGO,
  1, 0, 0,
  0, NULL,
  &irq_read_proc,
};


void led_on_handler(int irq, void *dev_id, struct pt_regs *regs) {
  /*
   * if this was a real device we would need to do some kind of 
   * interrupt_acknowledge here (disable hardware that generated this
   * interrupt), but here we don't have to do that because it's free running
   * anyway (expect to service this interrupt in much less time than 1/50
   * sec.)
   */
  if (led_on) {
    outb(0, LED_BASE);
  } else {
    outb(1,LED_BASE);
  }

  led_on = led_on == 0 ? 1 : 0;

}

int init_module(void) {

    int result = check_region(LED_BASE, 32);

    if (result) {
        printk(KERN_INFO "irq: can't get I/O address 0x%x\n", LED_BASE);
        return result;
    }
    request_region(LED_BASE, 32, "irq");

    /*
     * set up irq hadling
     */


    /*
     * request led on irq number and install handler
     */
    result = request_irq(IRQ_LED_ON, led_on_handler, 0, "LED on", NULL);
    if (result) {
      printk("<1>led_on_handler: failed install of irq %i handler...\n", IRQ_LED_ON);
      return result;
    }
    printk("<1>led_on_handler: irq %i handler installed...\n", IRQ_LED_ON);

    /*
     * request led off irq number and install handler
     */

    /*
     * register the proc fs inode
     */
    //proc_register_dynamic(&proc_root, &irq_proc_entry);
    outb(0x34,0x223);
    outb(0,0x220);
    outb(36,0x220);
    outb(0,0x221);
    outb(18,0x221);
    return 0;

}

void cleanup_module(void) {

    release_region(LED_BASE, 32);

    /*
     * unregister the proc fs inode
     */
    //proc_unregister(&proc_root, irq_proc_entry.low_ino);

    /*
     * release grabbed irq lines
     */
    free_irq(IRQ_LED_ON, NULL);

    printk("<1>irq removed...\n");

}

