#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <assert.h>

#include "qcam.h"
#include "qcam-os.h"
#include "qcam-os.c"		// i know it's ugly, but it's _fast_ ;)

/* Prototypes for static functions.  Externally visible functions
 * should be prototyped in qcam.h */

static int qc_handshake		(u_int val);
static int qc_handshake2	(u_int val);
static int qc_command		(int command);
static int qc_readparam		(void);

#define QC_HEIGHT	240
#define QC_WIDTH	240
#define QC_TRANSFER_SCALE 4

#define BPP	6
/* 0000 XXXX								*/
/*      ||||								*/
/*      |||+->	port_mode:	bidir   (unidir ... 0, bidir ... 1) 	*/
/*	||+-->	bpp:		6bpp	(4 ... 0, 6 ... 1)		*/
/*	++--->	scale:		4	(1 ... 0, 2 ... 1, 4 ... 2)	*/
/*									*/
#define QC_MODE	0x0F

// DENT: just for linux-test

#define	QC_PORT	0x378

/******************************************************************/

void qc_init (void)
{
  q->lp_control.data=0;
  q->lp_control.bit.strobe = 1;
  q->lp_control.bit.autofeed = 1;
  q->lp_control.bit.pcack = 1;

/**********/
/* reset */

  q->lp_control.bit.reset_n = 0;
  qc_handshake (1);

  q->lp_control.bit.reset_n = 1;
  qc_handshake (0);

/**********/

  qc_command (CMD_SetNumV);		qc_command (QC_HEIGHT / QC_TRANSFER_SCALE);
  qc_command (CMD_SetBrightness);	qc_command (130);
  qc_command (CMD_SetTop);		qc_command (1);
  qc_command (CMD_SetLeft);		qc_command (14/2);
  qc_command (CMD_BW_SetContrast);	qc_command (104);
  qc_command (CMD_BW_SetOffset);	qc_command (120);
  qc_command (CMD_SetNumH);		qc_command (((QC_WIDTH * QC_BPP) + (24 * QC_TRANSFER_SCALE)) / (24 * QC_TRANSFER_SCALE));
}

/******************************************************************/

void qc_open(void)
{
  if(enable_ports(q) == -1)
    return (1);
}

/******************************************************************/

void qc_close(void)
{
  disable_ports(q);
}

/******************************************************************/

void qc_command (int command)
{
  int hi, lo;
  int cmd;

  write_lpdata (command);
  qc_readparam ();
}

/******************************************************************/

static int qc_readparam (void)
{
  int hi, lo;
  int cmd;

  hi = qc_handshake (1);
  lo = qc_handshake (0);

  cmd = (hi & 0xf0) | ((lo & 0xf0) >> 4);

  return (cmd);
}

/******************************************************************/

static int qc_handshake (u_int val)
{
  u_int status=0;
  int i = 32000;

  q->lp_control.bit.pcack = ~val & 0x01;
  write_lpcontrol (q, q->lp_control.data);

  val <<= 3;
  while (i-- && (((status = read_lpstatus(q)) & 0x08) != val));

  return (i ? status : -1);
}

/******************************************************************/

static int qc_handshake2 (u_int val)
{
  u_int status;
  int i = 20000;

  q->lp_control.bit.pcack = ~val & 0x01;
  write_lpcontrol (q, q->lp_control.data);

  while (i-- && (((status = read_lpstatus_data(q)) & 0x01 ) != val));

  return (i ? status : -1);
}

/******************************************************************/

scanbuf *qc_scan (void)
{
  u_char *buffer;
  int i=0, j=0, k=0;
  int lines, pixels_per_line;
  u_int hi, hi2, lo, lo2;

  qc_command (CMD_SendVideoFrame); qc_command (QC_MODE);

/**********/

  q->lp_control.bit.bidir = 1;
  qc_handshake (1);
  qc_handshake (0);

/**********/

  lines = HEIGHT / TRANSFER_SCALE;
  pixels_per_line = WIDTH / TRANSFER_SCALE;

  buffer = malloc (lines * pixels_per_line * (int) (BPP/8. + 0.5));

  for (i = 0; i < lines * pixels_per_line; i+=4) {
    lo = qc_handshake2 (1);
    hi = (lo >> 8) & 0xff;
    lo = (lo >> 1) & 0x7f;

    lo2 = qc_handshake2 (0);
    hi2 = (lo2 >> 8) & 0xff;
    lo2 = (lo2 >> 1) & 0x7f;

    buffer[i+0] = lo & 0x3f;
    buffer[i+1] = ((lo & 0x40) >> 6) | ((hi & 0xf8) >> 2);
    buffer[i+2] = lo2 & 0x3f;
    buffer[i+3] = ((lo2 & 0x40) >> 6) | ((hi2 & 0xf8) >> 2);
  }

/**********/

  q->lp_control.bit.bidir = 0;
  qc_handshake (1);
  qc_handshake (0);

/**********/

  return (ret);
}
